=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2008-02-03 22:55:08 +0000
+++ bzrlib/graph.py	2008-02-25 21:47:01 +0000
@@ -424,6 +424,32 @@
                 raise errors.NoCommonAncestor(left_revision, right_revision)
             revisions = lca
 
+    def iter_ancestry(self, revision_id):
+        """Iterate the ancestry of this revision.
+
+        The specific order is undefined, but children should be returned before
+        parents.
+
+        :return: Yield tuples mapping a revision_id to its parents for the
+            ancestry of revision_id.
+        """
+        # XXX: Do we want to guarantee that children will be returned before
+        #      parents? At present that is the order, but I don't know that it
+        #      is beneficial to require it.
+        pending = set([revision_id])
+        processed = set()
+        while pending:
+            processed.update(pending)
+            next_map = self.get_parent_map(pending)
+            next_pending = set()
+            for item in next_map.iteritems():
+                yield item
+                next_pending.update(p for p in item[1] if p not in processed)
+            ghosts = pending.difference(next_map)
+            for ghost in ghosts:
+                yield (ghost, ())
+            pending = next_pending
+
     def iter_topo_order(self, revisions):
         """Iterate through the input revisions in topological order.
 

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2008-01-17 22:41:32 +0000
+++ bzrlib/tests/test_graph.py	2008-02-25 21:47:01 +0000
@@ -238,6 +238,22 @@
             'f':[NULL_REVISION]}
 
 
+# A graph that contains a ghost
+#  NULL_REVISION
+#       |
+#       f
+#       |
+#       e   g
+#      / \ /
+#     b   d
+#     | \ |
+#     a   c
+
+with_ghost = {'a': ['b'], 'c': ['b', 'd'], 'b':['e'], 'd':['e', 'g'],
+              'e': ['f'], 'f':[NULL_REVISION]}
+
+
+
 class InstrumentedParentsProvider(object):
 
     def __init__(self, parents_provider):
@@ -489,6 +505,21 @@
         self.assertFalse(graph.is_ancestor('a', 'c'))
         self.assertTrue('null:' not in instrumented_provider.calls)
 
+    def test_iter_ancestry(self):
+        graph = self.make_graph(boundary)
+        expected = boundary.copy()
+        expected.pop('a') # 'a' is not in the ancestry of 'c', all the
+                          # other nodes are
+        self.assertEqual(expected, dict(graph.iter_ancestry('c')))
+
+    def test_iter_ancestry_with_ghost(self):
+        graph = self.make_graph(with_ghost)
+        expected = with_ghost.copy()
+        # 'a' is not in the ancestry of 'c', and 'g' is a ghost
+        expected.pop('a') 
+        expected['g'] = ()
+        self.assertEqual(expected, dict(graph.iter_ancestry('c')))
+
     def test_filter_candidate_lca(self):
         """Test filter_candidate_lca for a corner case
 

